@@ -31,7 +31,7 @@ import Data.Default
 import Data.Functor
 import Data.List
 import qualified Data.Map as M hiding(null)
--- import Data.Maybe
+import Data.Maybe
 import Data.Version
 
 import Graphics.UI.Gtk.Abstract.Widget
@@ -47,12 +47,13 @@ import Paths_hbro
 
 import Prelude hiding(init)
 
+import System.Environment.XDG.BaseDir
 import System.Exit
 import System.Glib.Signals
 import System.Posix.Files
 import System.Posix.Signals
-import System.ZMQ3 (Rep(..))
-import qualified System.ZMQ3 as ZMQ
+import System.ZMQ4 (Rep(..))
+import qualified System.ZMQ4 as ZMQ
 -- }}}
 
 
@@ -64,54 +65,66 @@ hbro setup = do
 
     when (opts^.Options.help)      $ putStrLn Options.usage >> exitSuccess
     when (opts^.Options.version)   $ putStrLn (showVersion version) >> exitSuccess
-    when (opts^.Options.verbose)   . putStrLn $ "Commandline options: " ++ show opts
     when (opts^.Options.recompile) $ Dyre.recompile >>= maybe exitSuccess (\e -> putStrLn e >> exitFailure)
 
     Dyre.wrap hbro' opts (setup, opts)
 
 
 hbro' :: (K (), CliOptions) -> IO ()
-hbro' (customSetup, options) = do
+hbro' (customSetup, opts) = do
     void $ installHandler sigINT (Catch onInterrupt) Nothing
 
-    config <- runReaderT initConfig options
-    gui    <- runReaderT initGUI options
+    when (opts^.Options.verbose) . putStrLn $ "Commandline options: " ++ show opts ++ "\n"
 
-    (result, logs) <- withIPC options $ \ipc -> runK options config gui ipc $ main customSetup
+    config <- runReaderT initConfig opts
+    gui    <- initGUI opts
+
+    (result, logs) <- withIPC opts $ \ipc -> runK opts config gui ipc $ main customSetup
 
     either print return result
-    unless (options^.Options.quiet) . unless (null logs) $ putStrLn logs
-    unless (options^.Options.quiet) $ putStrLn "Exiting..."
+    unless (opts^.Options.quiet) . unless (null logs) $ putStrLn logs
+    unless (opts^.Options.quiet) $ putStrLn "Exiting..."
 
 -- {{{ Initialization
 initConfig :: (MonadBase IO m, OptionsReader m) => m (Config K)
 initConfig = do
     options <- readOptions id
-    return $ id
+    return . id
           . (options^.Options.quiet   ? set verbosity Quiet   ?? id)
           . (options^.Options.verbose ? set verbosity Verbose ?? id)
           $ def
 
 
-initGUI :: (MonadBase IO m, OptionsReader m) => m (GUI K)
-initGUI = do
-    file     <- Options.getUIFile
-    fallback <- io $ getDataFileName "examples/ui.xml"
+initGUI :: (MonadBase IO m) => CliOptions -> m (GUI K)
+initGUI opts = do
+    result <- runErrorT . (`runReaderT` opts) $ do
+        file <- getUIFile
+        whenM (readOptions Options.verbose) . io . putStrLn $ "Building GUI from file " ++ file ++ "\n"
 
-    file' <- io $ firstReadableOf [file, fallback]
+        io $ void GTK.initGUI
+        Gui.buildFrom file
+    either (\e -> io (putStrLn e >> exitFailure)) return result
 
-    case file' of
-        Just f -> do
-          io $ void GTK.initGUI
-          Gui.buildFrom f
-        _ -> io $ putStrLn "No UI file found." >> exitFailure
+
+getUIFile :: (MonadBase IO m, OptionsReader m, MonadError String m) => m FilePath
+getUIFile = do
+    fileFromOption  <- readOptions Options.uIFile
+    fileFromConfig  <- io (getUserConfigDir "hbro" >/> "ui.xml")
+    fileFromPackage <- io $ getDataFileName "examples/ui.xml"
+
+    maybe
+        (throwError "[ERROR] No readable UI file found.")
+        return
+        =<< io (firstReadableOf . catMaybes $ [fileFromOption, Just fileFromConfig, Just fileFromPackage])
   where
     firstReadableOf []    = return Nothing
     firstReadableOf (x:y) = do
-        isReadable <- fileAccess x True False False
+        isThere <- fileExist x
+        isReadable <- case isThere of
+            False -> return False
+            True -> fileAccess x True False False
         isReadable ? return (Just x) ?? firstReadableOf y
 
-
 withIPC :: (MonadBaseControl IO m) => CliOptions -> (IPC -> m a) -> m a
 withIPC options f = restoreM =<< (liftBaseWith $ \runInIO -> do
   ZMQ.withContext $ \c -> do
@@ -138,7 +151,7 @@ main customSetup = do
     bindTitleChanged      =<< Gui.readGUI Gui.webView
 
 -- Default web settings
-    WS.set webSettingsMonospaceFontFamily               "consolas"
+    WS.set webSettingsMonospaceFontFamily               "inconsolata"
     WS.set webSettingsEnableDeveloperExtras             True
     WS.set webSettingsEnableHtml5Database               False
     WS.set webSettingsEnableHtml5LocalStorage           False
@@ -221,17 +234,19 @@ bindNewWebView webView = do
         return webView'
 
 
-bindDownload :: (MonadBase IO m, MonadBaseControl IO m, ConfigReader m m, MonadError HError m) =>  WebView -> m ()
+bindDownload :: (MonadBase IO m, MonadBaseControl IO m, ConfigReader m m, NotificationReader m, MonadError HError m) =>  WebView -> m ()
 bindDownload webView = do
     void . liftBaseWith $ \runInIO -> on webView W.downloadRequested $ \d -> do
         amount <- W.downloadGetTotalSize d
 
-        --notify 5000 $ "Requested download: " ++ filename ++ " (" ++ show size ++ ")"
         void . runInIO $ do
             uri      <- downloadGetUri d
             filename <- downloadGetSuggestedFilename d
             callback <- readConfig onDownload
-            logV $ "Requested download <" ++ show uri ++ ">"
+
+            logV        $ "Requested download <" ++ show uri ++ ">"
+            notify 5000 $ "Requested download: " ++ filename ++ " (" ++ show amount ++ ")"
+
             callback uri filename amount
         return False
 
@@ -327,7 +342,7 @@ bindKeys = do
             case (Key.lookup allStrokes =<< M.lookup theMode theBindings) of
                 Just (Key.Leaf callback) -> Key.writeStatus Key.strokes [] >> callback `catchError` \e -> (io $ print e) >> notify 5000 (show e)
                 Just (Key.Branch _)      -> Key.writeStatus Key.strokes allStrokes
-                _                    -> return () --}
+                _                        -> return ()
             return ()
         _ -> return ()
     return False
