@@ -19,7 +19,6 @@ import Data.Map (Map)
 import qualified Data.Map as M
 
 import Graphics.UI.Gtk.Gdk.EventM
-import Graphics.UI.Gtk.WebKit.WebNavigationAction
 
 import Network.URI as N hiding(parseURI, parseURIReference)
 
@@ -51,8 +50,9 @@ data Config m = Config {
 makeLenses ''Config
 
 instance Show (Config m) where
-    show c = "Home page        = " ++ (show $ c^.homePage)
-        ++ "\nVerbosity        = " ++ (show $ c^.verbosity)
+    show c = unlines [
+        "Home page        = " ++ (show $ c^.homePage),
+        "Verbosity        = " ++ (show $ c^.verbosity)]
 
 
 -- | 'MonadReader' for 'Config'
@@ -72,17 +72,6 @@ type ConfigState n m = (ConfigReader n m, ConfigWriter n m)
 
 modifyConfig :: (ConfigState n m) => Simple Lens (Config n) a -> (a -> a) -> m ()
 modifyConfig l f = writeConfig l . f =<< readConfig l
-
-instance Eq NavigationReason where
-  a == b = (fromEnum a) == (fromEnum b)
-
-instance Show NavigationReason where
-  show WebNavigationReasonLinkClicked   = "Link clicked"
-  show WebNavigationReasonFormSubmitted = "Form submitted"
-  show WebNavigationReasonBackForward   = "Back/forward"
-  show WebNavigationReasonReload        = "Reload"
-  show WebNavigationReasonFormResubmitted = "Form resubmitted"
-  show WebNavigationReasonOther         = "Other"
 -- }}}
 
 -- | Run an action unless verbosity is 'Quiet'
